/****************************************************************
* Firmware of the TI TRF7970A Booster Pack CE TYPE B Rev. 1.0
*
* Copyright (C) 2012 Texas Instruments, Inc.
*
* DESCRIPTION: The TRF7970A is an integrated analog front end and
* data framing system for a 13.56 MHz NFC/RFID reader system.
* Built-in programming options make it suitable for a wide range
* of applications both in proximity and vicinity RFID systems.
* The reader is configured by selecting the desired protocol in
* the control registers. Direct access to all control registers
* allows fine tuning of various reader parameters as needed.
*
* The TRF7970A is interfaced to a MSP430G2553 microcontroller
* through the SPI (serial) interface using a hardware USART.
* The MCU is the master device and initiates all communication
* with the reader.
*
*
*
* AUTHOR:    Embedded RF    DATE: June 13, 2012
*
* CHANGES:
* REV.    DATE         WHO    DETAIL
* 1.0   June 13, 2012  ERF    Orginal Version
*
*
* BUILT WITH:
* IAR for MSP430, v5.40.1 (5.40.1.50328)
* (C) Copyright Texas Instruments, 2012. All rights reserved.
*****************************************************************
*****************************************************************
* All software and related documentation is provided "AS IS" and
* without warranty or support of any kind and Texas Instruments
* expressly disclaims all other warranties, express or implied,
* including, but not limited to, the implied warranties of
* merchantability and fitness for a particular purpose. Under no
* circumstances shall Texas Instruments be liable for any
* incidental, special or consequential damages that result from
* the use or inability to use the software or related
* documentation, even if Texas Instruments has been advised of
* the liability.
*
* Unless otherwise stated, software written and copyrighted by
* Texas Instruments is distributed as "freeware". You may use
* and modify this software without any charge or restriction.
* You may distribute to others, as long as the original author
* is acknowledged.
****************************************************************/
//
//===============================================================

#include <msp430.h>                         // processor specific header
#include <stdlib.h>                         // general purpose standard library
#include <stdio.h>                          // standard input/output header
#include "mcu.h"
#include "trf796x.h"
#include "types.h"
#include "spi.h"

void I2c_setup(void);
//===============================================================

u08_t buf[210];
u08_t enable = 0;
u08_t i_reg = 0x01;                            // interrupt register
u08_t irq_flag = 0x00;
u08_t rx_error_flag = 0x00;
s08_t rxtx_state = 1;                            // used for transmit receive byte count
u08_t i2c_flag = 0;
unsigned char *PRxData;                     // Pointer to RX data
unsigned char RXByteCtr;
unsigned char RxBuffer[12];       // Allocate 12 byte of RAM

//===============================================================
//
//===============================================================
//
// Macro definitions
//

/*! Module is busy waiting for IRQ */
#define INT_BUSY            BIT0
/*! RX detected and in buffer */
#define INT_RX_COMPLETE     BIT1
/*! Critical error during RFID processing */
#define INT_CRITICAL_ERROR  BIT2
/*! Communication error (protocol error) */
#define INT_COMM_ERROR      BIT7
/*! TX Completed */
#define INT_TX_COMPLETE     BIT3


/*! Interrupt, error, flow flags */
u08_t u8InterruptFlags;

#define IRQ_IS_SET()            (P2IN & IRQ_PIN)
#define SLAVESELECT_HIGH()      {SPI_SS_OUT |= SPI_SS_PIN;}
#define SLAVESELECT_LOW()       {SPI_SS_OUT &= ~SPI_SS_PIN;}

typedef enum
{
    POWER_OFF_STATE= 0,     /* Default state after reset */
    SENSE_STATE,            /* Target is waiting for 14443B transponder emulation command from reader */
    EMULATION_STATE,        /* Target is in card emulation mode */
    NDEF_MESSAGE_STATE,      /* Target has sent the NDEF Message */
    DESELECTED_STATE        /* Target has been deselected from the reader */
}
NFC_state_t;
NFC_state_t NFC_State;

#define FIFO_LEN	127

typedef enum
{
    READER_STATE = 0,
    TAG_STATE
}TRF_state_t;

// The TRF7970 will initially be a tag for 1 second, then switch to the Reader State for another second
TRF_state_t card_state = TAG_STATE;

u08_t state_250ms_counter = 0;

/*! Buffer pointer */
static u08_t * buf_ptr;



/******************************************************************************
 * @brief  Protocol Error handler
 *  Called when the TRF7970 detects a protocol error
 *
 * @return    None
 *****************************************************************************/
void NFC_IRQ_PROTOCOL_ERR(void)
{
    u8InterruptFlags = INT_COMM_ERROR;
    Trf796xInitialSettings();
}

/******************************************************************************
 * @brief  TX handler
 *  Called when the TRF7970 detects a Transmission complete
 *
 * @return    None
 *****************************************************************************/
void  NFC_IRQ_TX(void)
{
    // Reset FIFO with an extra dummy clock after last byte has been read out
    buf[0] = 0x00;
    RADIO_WRITERAW(RESET|BIT7, buf,1);
    u8InterruptFlags = INT_TX_COMPLETE;
}

/******************************************************************************
 * @brief  RX handler
 *  Called when the TRF7970 detects a data Reception
 *  It puts the data in buffers which will be passed to upper layers
 *
 * @param Fifo_Int 1: Interrupted by FIFO, 0: RX complete
 *
 * @return    None
 *****************************************************************************/
void  NFC_IRQ_RX(u08_t Fifo_Int)
{
    u08_t Reg;

    buf_ptr = &buf[0];
    RADIO_READSINGLE(FIFO_STATUS, &Reg); 		//determine the number of bytes left in FIFO
    Reg = 0x7F & Reg;

    // Get all bytes from FIFO and put them in the buffer
    RADIO_READCONT(FIFO, buf_ptr, (Reg));
    u8InterruptFlags = INT_RX_COMPLETE;
}

/******************************************************************************
 * @brief Process a TRF797x IRQ
 * Called when an IRQ is detected, calls the appropiate function from NFC layer
 *
 * @param IRQ_Stat value read from IRQ register
 *
 * @return  none
 *****************************************************************************/
void Radio_Process_IRQ(u08_t IRQ_Stat)
{

     if (IRQ_Stat == (BIT7 | BIT5))
    {
         //NFC_IRQ_FIFO();
    }
    else if (IRQ_Stat & BIT0)
    {
//        NFC_IRQ_CRITICAL_ERR();
    }
    else if (IRQ_Stat & BIT4)
    {
        NFC_IRQ_PROTOCOL_ERR();
    }
    else if (IRQ_Stat & BIT2)
    {
//        NFC_IRQ_RF();
    }
    else if (IRQ_Stat & BIT3)
    {
        // IRQ SDD interrupt
//        NFC_IRQ_SDD();
    }
     else if (IRQ_Stat & BIT6)
    {
        // Reception detected, pass if FIFO RX or end of RX
         NFC_IRQ_RX(IRQ_Stat & BIT5);
    }
     else if (IRQ_Stat & BIT7)
    {
         NFC_IRQ_TX();
    }
    else
    {
    }
}

/******************************************************************************
 * @brief Waits for a radio IRQ
 * This function will lock the mcu while waiting for IRQ. Note that the
 * microcontroller is not expected to do anything else in BSL
 *
 * @return  none
 *****************************************************************************/
void Radio_Wait_IRQ(void)
{
    u08_t Register[4];


   // Wait for an IRQ forever
    while (!IRQ_IS_SET() && card_state == TAG_STATE );



    do
    {
        if ( card_state == TAG_STATE)
        {
          // Clear Buffer
         Register[0] = IRQ_STATUS;
         Trf796xReadCont(Register,2);    /* IRQ status register address */

         Register[2] = NFC_TARGET_PROTOCOL;
         Trf796xReadSingle(&Register[2],1);

        if(Register[2] == 0xC5)
         {
           Radio_Process_IRQ(Register[0]);
           // Make sure card_state = TAG_STATE;
           card_state = TAG_STATE;
           // Stop watchdog timer
           WDTCTL = WDTPW + WDTHOLD;
           break;
         }
         else if (((Register [2])&0x0F) != 0x00 || Register [2] == 0xC0)   //|| Register [2] == 0x80)
         {
           Trf796xInitialSettings();
           NFC_State = SENSE_STATE;
         }
         else
           NFC_State = SENSE_STATE;
         }
        else
        {
        }

    }while (IRQ_IS_SET() && card_state == TAG_STATE);

    return;
}

void NFCSend(u08_t  *pbuf)
{
    u08_t total_length, length, i;
    u08_t tx_buf[FIFO_LEN+3];

    u8InterruptFlags = INT_BUSY;


    total_length = *pbuf;
    length = total_length;
    if(length > FIFO_LEN)
      length = FIFO_LEN;
	
    // First, clear FIFO with a reset
    tx_buf[0] = TRANSMIT_CRC | BIT7;  // TX command
    tx_buf[1] = TX_LENGTH_BYTE_1 | BIT5;   // Write cont TXLength
    tx_buf[2] = (u08_t)(total_length >> 4);
    tx_buf[3] = (u08_t)(total_length << 4);

    for (i=1; i <= length ; i++)
         tx_buf[i+3] = pbuf[i];

    RADIO_WRITERAW(RESET | BIT7, tx_buf, length+4);

    do{
        // Wait for TX Complete interrupt
        Radio_Wait_IRQ();
    }while(u8InterruptFlags != INT_TX_COMPLETE);

}

u08_t NFC_ProcessReceivedData(u08_t * buffer)
{
    u16_t command;
    u08_t ret = 0x00; //RX_PACKET_ONGOING
    command = buffer[0];
    switch (command)
    {
        case 0x05:
          // Reset FIFO with an extra dummy clock
          buffer[0] = 0x00;
           RADIO_WRITERAW(RESET|BIT7, buffer,1);

          // ATQB Packet
          buffer[0] = 12;         // Length
          buffer[1] = 0x50;       // ATQB Start Byte
          // NFCID 3:0
          buffer[2] = TargetCID[0];
          buffer[3] = TargetCID[1];
          buffer[4] = TargetCID[2];
          buffer[5] = TargetCID[3];

          // Application Data Bytes
          buffer[6] = 0x40;     // AFI
          buffer[7] = 0xE2;     // CRC_B
          buffer[8] = 0xAF;     // CRC_B
          buffer[9] = 0x11;     // # of applications (1)

          // Protocol Info Bytes
          buffer[10] = 0x00;    // Date Rate Capability ( Only Support 106 kbps)
          // Max Frame/Protocol type (128 bytes / PICC compliant to -4)
          buffer[11] = 0x71;
          // (FWI/ADC/FO) ( FWT = 77.3mSec, ADC = coded according to AFI, CID supported)
          buffer[12] = 0x85;

          NFCSend(&buffer[0]);		//Send ATQB Packet and wait for TX Complete

          break;
        case 0x1D:
          if( buffer[1] == TargetCID[0] &&
              buffer[2] == TargetCID[1] &&
              buffer[3] == TargetCID[2] &&
              buffer[4] == TargetCID[3] )
          {
            // Reset FIFO with an extra dummy clock
            buffer[0] = 0x00;
            RADIO_WRITERAW(RESET|BIT7, buffer,1);

            // ATTRIB Response
            buffer[0] = 0x01; // Length

            NFCSend(&buffer[0]);		//Send ATTRIB Response and wait for TX Complete

          }

          break;
        case 0xB2:
           // Reset FIFO with an extra dummy clock
           buffer[0] = 0x00;
           RADIO_WRITERAW(RESET|BIT7, buffer,1);
           buffer[0] = 1;
           buffer[1] = 0xA2;
           NFCSend(&buffer[0]);
          break;

        case 0xC2: // deselect, section 8.8
           //Done reading tag so clear P2.5 to prep for next read attempt.
           //BT_WAKE_OFF;
           // Reset FIFO with an extra dummy clock
           buffer[0] = 0x00;
           RADIO_WRITERAW(RESET|BIT7, buffer,1);

           if ( NFC_State == NDEF_MESSAGE_STATE)
           {
             buffer[0] = 3;
             buffer[1] = 0x02;
             buffer[2] = 0x90;
             buffer[3] = 0x00;
             NFCSend(&buffer[0]);
             NFC_State = DESELECTED_STATE;
           }
           else
           {
             buffer[0] = 1;
             buffer[1] = 0x02;
             NFCSend(&buffer[0]);
           }

           break;

        case 0x02:
          if(buffer[2] == 0xA4) // container capability select command, section 8.2, figure 14 command#1
          {
            // Reset FIFO with an extra dummy clock
            buffer[0] = 0x00;
            RADIO_WRITERAW(RESET|BIT7, buffer,1);
            buffer[0] = 0x03;   // Length
            buffer[1] = 0x02;
            buffer[2] = 0x90;
            buffer[3] = 0x00;
            NFCSend(&buffer[0]); // sends container capability select response, figure 15

            //tag is being read so wake up Bluetooth chip and put it in connectable mode
            P2OUT |= BIT5;                          // Toggle P2.5 using exclusive-OR
          }
          else if( buffer[2] == 0xB0 && buffer[5] == 0x0F) // read binary command, section 8.3, figure 16, command#3
          {
            // Reset FIFO with an extra dummy clock
            buffer[0] = 0x00;
            RADIO_WRITERAW(RESET|BIT7, buffer,1);
            buffer[0] = 18;   // Length
            buffer[1] = 0x02;
            buffer[2] = 0x00;
            buffer[3] = 0x0F;
            buffer[4] = 0X20;
            buffer[5] = 0X00;
            buffer[6] = 0x50;  //max R-APDU size
            buffer[7] = 0X00;
            buffer[8] = 0x50;  //max C-APDU size
            buffer[9] = 0X04;
            buffer[10] = 0X06;
            buffer[11] = 0XE1; // 11 and 12 set file identifier
            buffer[12] = 0x04;
            buffer[13] = 0x08; // 13 and 14 are setting MAX NDEF file size
            buffer[14] = 0x00;
            buffer[15] = 0X00; //read access without security
            buffer[16] = 0xff; //no write access;
            buffer[17] = 0X90;
            buffer[18] = 0X00;
            NFCSend(&buffer[0]); // sends read binary response, figure 17
          }
          else if( buffer[2] == 0xB0 && buffer[5] == 0x02) // read binary command, section 8.5, figure 20, command#5
          {
            // Reset FIFO with an extra dummy clock
            buffer[0] = 0x00;
            RADIO_WRITERAW(RESET|BIT7, buffer,1);
            buffer[0] = 5;   // Length
            buffer[1] = 0x02;
            buffer[2] = 0x00;
            buffer[3] = 0x2D; //0x2D for simple response, 0x3e for static handover // length of ndef message
            //buffer[3] = 0x3E; //0x2D for simple response, 0x3e for static handover // length of ndef message
            buffer[4] = 0x90;
            buffer[5] = 0x00;
            NFCSend(&buffer[0]); // sends read binary response, figure 21
          }
          else if( buffer[2] == 0xB0 && buffer[5] == 0x2D) // read binary command, section 8.7. buffer[5] must match length above, command#7
          {
            // Reset FIFO with an extra dummy clock
            buffer[0] = 0x00;
            RADIO_WRITERAW(RESET|BIT7, buffer,1);
            // Ready to send NDEF Message
            buffer[0] = 0x30; // length+3
            buffer[1] = 0x02;
            buffer[2] = 0x00; // 2 and 3 set (NDEF message length - 2)
            buffer[3]=	0x2B;
            buffer[4] = 0xD2; //start of NDEF message
            buffer[5] = 0x20;
            buffer[6] = 0x08;
            buffer[7] = 0x61;
            buffer[8] = 0x70;
            buffer[9] = 0x70;
            buffer[10] = 0x6C;
            buffer[11] = 0x69;
            buffer[12] = 0x63;
            buffer[13] = 0x61;
            buffer[14] = 0x74;
            buffer[15] = 0x69;
            buffer[16] = 0x6F;
            buffer[17] = 0x6E;
            buffer[18] = 0x2F;
            buffer[19] = 0x76;
            buffer[20] = 0x6E;
            buffer[21] = 0x64;
            buffer[22] = 0x2E;
            buffer[23] = 0x62;
            buffer[24] = 0x6C;
            buffer[25] = 0x75;
            buffer[26] = 0x65;
            buffer[27] = 0x74;
            buffer[28] = 0x6F;
            buffer[29] = 0x6F;
            buffer[30] = 0x74;
            buffer[31] = 0x68;
            buffer[32] = 0x2E;
            buffer[33] = 0x65;
            buffer[34] = 0x70;
            buffer[35] = 0x2E;
            buffer[36] = 0x6F;
            buffer[37] = 0x6F;
            buffer[38] = 0x62;
            buffer[39] = 0x08;
            buffer[40] = 0x00;
            /*
            buffer[41] = RxBuffer[0];	//the bdaddr lsb first;
            buffer[42] = RxBuffer[1];
            buffer[43] = RxBuffer[2];
            buffer[44] = RxBuffer[3];
            buffer[45] = RxBuffer[4];
            buffer[46] = RxBuffer[5]; // msb of address, end of NDEF message
            */
            buffer[41] = 0x8F;	//the bdaddr lsb first;
            buffer[42] = 0x10;
            buffer[43] = 0xAF;
            buffer[44] = 0xFE;
            buffer[45] = 0xFF;
            buffer[46] = 0xB8; // msb of address, end of NDEF message

            buffer[47] = 0x90;
            buffer[48] = 0x00;
            NFCSend(&buffer[0]); // sends read binary response, figure 27
          }
          else if( buffer[2] == 0xB0 && buffer[5] == 0x3E)
          {
            // Reset FIFO with an extra dummy clock
        	buffer[0] = 0x00;
        	RADIO_WRITERAW(RESET|BIT7, buffer,1);
            // Ready to send NDEF Message
            buffer[0] = 0x41; // length+3
            buffer[1] = 0x02;
            buffer[2] = 0x00; // 2 and 3 set (NDEF message length - 2)
            buffer[3]=	0x3C;
            buffer[4] = 0x91; //start of NDEF message
        	buffer[5] = 0x02;
        	buffer[6] = 0x0A;
        	buffer[7] = 0x48;
        	buffer[8] = 0x73;
        	buffer[9] = 0x12;
        	buffer[10] = 0xD1;
        	buffer[11] = 0x02;
        	buffer[12] = 0x04;
        	buffer[13] = 0x61;
        	buffer[14] = 0x63;
        	buffer[15] = 0x03;
        	buffer[16] = 0x01;
        	buffer[17] = 0x30;
        	buffer[18] = 0x00;
        	buffer[19] = 0x5A;
        	buffer[20] = 0x20;
        	buffer[21] = 0x08; // payload length must match buffer[56]
        	buffer[22] = 0x01;
        	buffer[23] = 0x61;
        	buffer[24] = 0x70;
        	buffer[25] = 0x70;
        	buffer[26] = 0x6C;
        	buffer[27] = 0x69;
        	buffer[28] = 0x63;
        	buffer[29] = 0x61;
        	buffer[30] = 0x74;
        	buffer[31] = 0x69;
        	buffer[32] = 0x6F;
        	buffer[33] = 0x6E;
        	buffer[34] = 0x2F;
        	buffer[35] = 0x76;
        	buffer[36] = 0x6E;
        	buffer[37] = 0x64;
        	buffer[38] = 0x2E;
        	buffer[39] = 0x62;
        	buffer[40] = 0x6C;
        	buffer[41] = 0x75;
        	buffer[42] = 0x65;
        	buffer[43] = 0x74;
        	buffer[44] = 0x6F;
        	buffer[45] = 0x6F;
        	buffer[46] = 0x74;
        	buffer[47] = 0x68;
        	buffer[48] = 0x2E;
        	buffer[49] = 0x65;
        	buffer[50] = 0x70;
        	buffer[51] = 0x2E;
        	buffer[52] = 0x6F;
        	buffer[53] = 0x6F;
        	buffer[54] = 0x62;
        	buffer[55] = 0x30;
        	buffer[56] = 0x08;
        	buffer[57] = 0x00;
        	buffer[58] = RxBuffer[0];	//the bdaddr lsb first;
        	buffer[59] = RxBuffer[1];
        	buffer[60] = RxBuffer[2];
        	buffer[61] = RxBuffer[3];
        	buffer[62] = RxBuffer[4];
        	buffer[63] = RxBuffer[5]; // msb of address, end of NDEF message
        	buffer[64] = 0x90;
        	buffer[65] = 0x00;
        	NFCSend(&buffer[0]); // sends read binary response, figure 27
          }
          break;
        case 0x03:
          if(buffer[2] == 0xA4 && buffer[7] == 0x03) //command#2
          {
            // Reset FIFO with an extra dummy clock
            buffer[0] = 0x00;
            RADIO_WRITERAW(RESET|BIT7, buffer,1);
            buffer[0] = 0x03;   // Length
            buffer[1] = 0x03;
            buffer[2] = 0x90;
            buffer[3] = 0x00;
            NFCSend(&buffer[0]);
          }
          else if (buffer[2] == 0xA4 && buffer[7] == 0x04) // NDEF select command section 8.4 Figure 18 command#4 and section 8.6 command#6
          {
            // Reset FIFO with an extra dummy clock
            buffer[0] = 0x00;
            RADIO_WRITERAW(RESET|BIT7, buffer,1);
            buffer[0] = 0x03;   // Length
            buffer[1] = 0x03;
            buffer[2] = 0x90;
            buffer[3] = 0x00;
            NFCSend(&buffer[0]); // sends NDEF select response figure 19
          }
          else if (buffer[2] == 0xB0 ) //command#8
          {
            // Reset FIFO with an extra dummy clock
            buffer[0] = 0x00;
            RADIO_WRITERAW(RESET|BIT7, buffer,1);
            buffer[0] = 0x05;   // Length
            buffer[1] = 0x03;
            buffer[2] = 0x2F;
            buffer[3] = 0x20;
            buffer[4] = 0x90;
            buffer[5] = 0x00;
            NFCSend(&buffer[0]);
            NFC_State = NDEF_MESSAGE_STATE;
          }
          break;
    }

    return ret;
}

NFC_state_t NFCTarget_statemachine(void)
{
    switch (NFC_State)
    {
        case POWER_OFF_STATE:
           Trf796xInitialSettings();
           NFC_State = SENSE_STATE;

           break;
        case SENSE_STATE:

            NFC_State = SENSE_STATE;

            u8InterruptFlags = INT_BUSY;
            do{
              Radio_Wait_IRQ();
            }while(u8InterruptFlags != INT_RX_COMPLETE && card_state == TAG_STATE );


            if(u8InterruptFlags == INT_RX_COMPLETE)
            {
              NFC_State = EMULATION_STATE;
            }

            else if (NFC_State != EMULATION_STATE )
              break;


        case EMULATION_STATE:
 	     // Now wait until the part is selected
         // restart the pointer and wait for a message
          do
          {
            if ((u8InterruptFlags &
              (INT_RX_COMPLETE | INT_CRITICAL_ERROR | INT_COMM_ERROR))
              ==  INT_RX_COMPLETE)
                NFC_ProcessReceivedData(&buf[0]); // Check if the message was a ATR_REQ
            else if ((u8InterruptFlags&INT_CRITICAL_ERROR))
              NFC_State = POWER_OFF_STATE;    // Timeout Error

            if( NFC_State == DESELECTED_STATE)
            {

              break;
            }

            do{
            // Wait for RX Complete interrupt
              Radio_Wait_IRQ();
            }while(u8InterruptFlags != INT_RX_COMPLETE && (NFC_State == EMULATION_STATE || NFC_State == NDEF_MESSAGE_STATE) );
          }
          while(NFC_State == EMULATION_STATE || NFC_State == NDEF_MESSAGE_STATE);
          __no_operation();

          if( NFC_State == SENSE_STATE)
            // Re-enable Watchdog timer
            WDTCTL = WDT_ADLY_250;

          break;
        case NDEF_MESSAGE_STATE:
          u8InterruptFlags = INT_BUSY;
          do{
             Radio_Wait_IRQ();
          }while(NFC_State != SENSE_STATE);
          NFC_State = SENSE_STATE;
        case DESELECTED_STATE:
           NFC_State = SENSE_STATE;
           do{
             Radio_Wait_IRQ();
          }while(NFC_State != SENSE_STATE);
          // Re-enable Watchdog timer
          WDTCTL = WDT_ADLY_250;
          break;

    }
    return NFC_State;
}

void I2c_setup(void)
{

	  P1SEL |= BIT6 + BIT7;                     // Assign I2C pins to USCI_B0
	  P1SEL2|= BIT6 + BIT7;                     // Assign I2C pins to USCI_B0
	  UCB0CTL1 |= UCSWRST;                      // Enable SW reset
	  UCB0CTL0 = UCMODE_3 + UCSYNC;             // I2C Slave, synchronous mode
	  UCB0I2COA = 0x48>>1;                         // Own Address is 048h
	  UCB0CTL1 &= ~UCSWRST;                     // Clear SW reset, resume operation
	  UCB0I2CIE |= UCSTPIE + UCSTTIE;           // Enable STT and STP interrupt
	  IE2 |= UCB0RXIE;                          // Enable RX interrupt

	  while (!(i2c_flag))
	  {
	    PRxData = (unsigned char *)RxBuffer;    // Start of RX buffer
	    RXByteCtr = 0;                          // Clear RX byte count
	    __bis_SR_register(CPUOFF + GIE);        // Enter LPM0 w/ interrupts
	                                            // Remain in LPM0 until master
	                                            // finishes TX
	    __no_operation();                       // Set breakpoint >>here<< and read RxBuffer
	  }
}



void main(void)
{
  WDTCTL = WDTPW + WDTHOLD;                 // Stop WDT
  // set up i2c interface on USCIB0 and get BDADDR before continuing.
  //I2c_setup();

  // Stop WDT
  WDTCTL = WDT_ADLY_250;                    // WDT 250ms, ACLK, interval timer
  IE1 |= WDTIE;                             // Enable WDT interrupt

  TRF_DISABLE;
  // Set EN to output direction
  ENABLE_SET;
  // Set EN = 1 on the TRF 79xx
  TRF_ENABLE;
  //BT_WAKE signal config and initialization
  P2DIR |= BIT5;                            // Set P2.5 to output direction

  // wait until system clock started
  McuDelayMillisecond(1);
  IRQ_PIN_SET;
  // set the oscillator
  McuOscSel(1);
  Trf796xReaderSettings();
  // settings for communication with TRF
  Trf796xCommunicationSetup();



  // ----->  Set MCU Clock Frequency to 8 MHz and OOK Modulation

  // Re-configure the USART with this external clock
  Trf796xReConfig();

  // General enable interrupts
  __bis_SR_register(GIE);

  OOK_DIR_IN;                            // set OOK port tristate
  enable = 1;                            // indicates, that setting are done


  for(;;)
  {
	  // Initialize TRF7970 in Card Emulation
	  NFCTarget_statemachine();
  }
}

// Watchdog Timer interrupt service routine
#pragma vector=WDT_VECTOR
__interrupt void watchdog_timer(void)
{
   __bic_SR_register_on_exit(LPM3_bits);  //exit LPM3
}

//------------------------------------------------------------------------------
// The USCI_B0 data ISR is used to move received data from the I2C master
// to the MSP430 memory.
//------------------------------------------------------------------------------
#pragma vector = USCIAB0TX_VECTOR
__interrupt void USCIAB0TX_ISR(void)
{
  *PRxData++ = UCB0RXBUF;                   // Move RX data to address PRxData
  RXByteCtr++;                              // Increment RX byte count
  if(RXByteCtr == 6)
	  i2c_flag = 1;
}

//------------------------------------------------------------------------------
// The USCI_B0 state ISR is used to wake up the CPU from LPM0 in order to
// process the received data in the main program. LPM0 is only exit in case
// of a (re-)start or stop condition when actual data was received.
//------------------------------------------------------------------------------
#pragma vector = USCIAB0RX_VECTOR
__interrupt void USCIAB0RX_ISR(void)
{
  UCB0STAT &= ~(UCSTPIFG + UCSTTIFG);       // Clear interrupt flags
  if (RXByteCtr)                            // Check RX byte counter
  {
    __bic_SR_register_on_exit(CPUOFF);      // Exit LPM0 if data was received
  }
}
