/****************************************************************
* FILENAME: trf796x.c
*
* BRIEF: Contains functions to initialize and execute
* communication the TRF796x via the selected interface.
*
* Copyright (C) 2010 Texas Instruments, Inc.
*
* AUTHOR(S): Reiser Peter        DATE: 02 DEC 2010
*
* EDITED BY:
* *
*
****************************************************************/

#include "trf796x.h"

//===============================================================

u08_t    command[2];
u08_t    direct_mode = 0;


extern u08_t    i_reg;

extern u08_t    irq_flag;
extern u08_t    rx_error_flag;
extern s08_t    rxtx_state;
extern u08_t    stand_alone_flag;

u08_t TargetCID[4] = TARGET_CID;


//===============================================================

void Trf796xISR(u08_t *irq_status);

//===============================================================
//                                                              ;
//===============================================================

void
Trf796xCommunicationSetup(void)
{
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    SpiSetup();
  }
  
}

//===============================================================
// NAME: void Trf796xDirectCommand (u08_t *pbuf)
//
// BRIEF: Is used to transmit a Direct Command to TRF796x.
//
// INPUTS:
//    Parameters:
//        u08_t        *pbuf        Direct Command
//
// OUTPUTS:
//
// PROCESS:    [1] transmit Direct Command
//
// CHANGE:
// DATE          WHO    DETAIL
// 24Nov2010    RP    Original Code
//===============================================================

void
Trf796xDirectCommand(u08_t *pbuf)
{
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    SpiDirectCommand(pbuf);
  }
  
}

//===============================================================
// NAME: void Trf796xDirectMode (void)
//
// BRIEF: Is used to start Direct Mode.
//
// INPUTS:
//
// OUTPUTS:
//
// PROCESS:    [1] start Direct Mode
//
// NOTE: No stop condition
//
// CHANGE:
// DATE          WHO    DETAIL
// 24Nov2010    RP    Original Code
//===============================================================

void
Trf796xDirectMode(void)
{
  direct_mode = 1;
  
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    SpiDirectMode();
  }
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf796xDisableSlotCounter(void)
{
  buf[40] = IRQ_MASK;
  buf[41] = IRQ_MASK;                // next slot counter
  Trf796xReadSingle(&buf[41], 1);
  buf[41] &= 0xFE;                // clear BIT0 in register 0x01
  Trf796xWriteSingle(&buf[40], 2);
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf796xEnableSlotCounter(void)
{
  buf[40] = IRQ_MASK;
  buf[41] = IRQ_MASK;                // next slot counter
  Trf796xReadSingle (&buf[41], 1);
  buf[41] |= BIT0;                // set BIT0 in register 0x01
  Trf796xWriteSingle(&buf[40], 2);
}




//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================


void Radio_Transfer(u08_t mode_addr, u08_t *pbuf, u08_t length)
{
    u16_t i;
    u08_t *txbuf, *rxbuf;
        
    for (i=length; i > 0; i--)
    {
        if (!(mode_addr & BIT6))    // If writing, readjust buff
           pbuf[i] = pbuf[i-1];
        else if (i != 0)          // When reading, clear buffer
          pbuf[i] = 0x00;
    }
    pbuf[0] = mode_addr;

    rxbuf = txbuf = pbuf; // Initialize pointer

    SLAVE_SELECT_LOW; //Start SPI Mode
    
     // USCI_A0 TX buffer ready?
    while (!(IFG2 & UCA0TXIFG));
    
    // Previous data to TX, RX
    UCA0TXBUF = *txbuf++;
    
    while(UCA0STAT & UCBUSY);    // Ensure USCI is not busy

    while(length > 0)
    {
        UCA0TXBUF = *txbuf++;
    
        while(UCA0STAT & UCBUSY);    // Ensure USCI is not busy

        *rxbuf++ = UCA0RXBUF;
        length--;
    }
    
    while(UCA0STAT & UCBUSY);    // Ensure USCI is not busy

    SLAVE_SELECT_HIGH; //Stop SPI Mode
}

void Radio_InitialSettings(void)
{
   RADIO_DIRECTCOMMAND(SOFT_INIT, buf);
}

void
Trf796xReaderSettings(void)
{
  command[0] = SOFT_INIT;
  Trf796xDirectCommand(command);
  
  command[0] = IDLE;
  Trf796xDirectCommand(command);
}

void
Trf796xInitialSettings(void)
{
	unsigned int i;
    
    // Software Init
    RADIO_DIRECTCOMMAND(SOFT_INIT, buf);

    // Idle 
    RADIO_DIRECTCOMMAND(IDLE, buf);
    
    // Mod Sysclock
    buf[0] = 0x00;
    RADIO_WRITESINGLE(MODULATOR_CONTROL, buf);  // ASK 10%

    // 106 kbps Tag Emulator 14443B mode
    buf[0] = 0x25; // Type B		
    RADIO_WRITESINGLE(ISO_CONTROL, buf);
    
    // Regulator & IO Control
    buf[0] = 0x87;
    RADIO_WRITESINGLE(REGULATOR_CONTROL,buf);
    
    // RX Special Settings 
    buf[0] = 0x3C;
    RADIO_WRITESINGLE(RX_SPECIAL_SETTINGS, buf);
    
    // NFC Target Detection Level (NFCID size = 4)
    buf[0] = BIT2 + BIT1 + BIT0;     // Type B
    RADIO_WRITESINGLE(NFC_TARGET_LEVEL, buf);
    
    // NFCID
    for ( i=0; i < 4; i++)
        buf[i] = TargetCID[i];
    RADIO_WRITECONT(NFCID, &buf[0], 4);
    
    //Low Level Field Detection
    buf[0] = BIT1+BIT0;  //0x03 // Type B Card
    RADIO_WRITESINGLE(NFCLowDetection, buf);
    
    // ISO 14443B Options
    buf[0] = 0x00;
    RADIO_WRITESINGLE(ISO_14443B_OPTIONS, buf);
    
    // NFC Tag 
    buf[0] = 0x20;
    RADIO_WRITESINGLE(CHIP_STATE_CONTROL, buf);  
    
    // Reset FIFO with an extra dummy clock
    buf[0] = 0x00;
    RADIO_WRITERAW(RESET|BIT7, buf,1);
    
    // Block Receiver, then Enable Receiver
       //--Resetting Decoders--//
    RADIO_DIRECTCOMMAND(STOP_DECODERS, buf); 
    RADIO_DIRECTCOMMAND(RUN_DECODERS, buf);
}

//===============================================================
// The Interrupt Service Routine determines how the IRQ should  ;
// be handled. The TRF796x IRQ status register is read to       ;
// determine the cause of the IRQ. Conditions are checked and   ;
// appropriate actions taken.                                   ;
//===============================================================

void Trf796xISR(u08_t *irq_status)
{
  
  if(*irq_status == 0xA0)            // BIT5 and BIT7
  {                                // TX active and only 3 bytes left in FIFO
    i_reg = 0x00;
    
  }
  
  else if(*irq_status == BIT7)
  {                                // TX complete
    i_reg = 0x01;
    Trf796xReset();                // reset the FIFO after TX
    
  }
  else if((*irq_status & BIT1) == BIT1)
  {                                // collision error
    i_reg = 0x02;                // RX complete
    
    
    Trf796xStopDecoders();                        // reset the FIFO after TX
    
    Trf796xReset();
    
    Trf796xResetIrqStatus();
    
    IRQ_CLR;
  }
  else if(*irq_status == BIT6)
  {    // RX flag means that EOF has been received
    // and the number of unread bytes is in FIFO status regiter
    if(rx_error_flag == 0x02)
    {
      i_reg = 0x02;
      return;
    }
    
    *irq_status = FIFO_STATUS;
    Trf796xReadSingle(irq_status, 1);            // determine the number of bytes left in FIFO
    *irq_status = (0x0F &*irq_status);
    buf[rxtx_state] = FIFO;                // write the received bytes to the correct place of the buffer
    
    
    Trf796xReadCont(&buf[rxtx_state], *irq_status);
    rxtx_state = rxtx_state +*irq_status;
    
    if((*irq_status & BIT0) == BIT0)
    {
      *irq_status = (*irq_status >> 1) & 0x07;    // mask the first 5 bits
      *irq_status = 8 -*irq_status;
      buf[rxtx_state - 1] &= 0xFF << *irq_status;
    }                                // if

    i_reg = 0xFF;                    // signal to the receive function that this are the last bytes
  }
  else if((*irq_status & BIT4) == BIT4)                // CRC error
  {
    if((*irq_status & BIT5) == BIT5)
    {
      i_reg = 0x01;    // RX active
      rx_error_flag = 0x02;
    }
    if((*irq_status & BIT6) == BIT6)            // 4 Bit receive
    {
      buf[200] = FIFO;                        // write the recieved bytes to the correct place of the buffer
      
      Trf796xReadCont(&buf[200], 1);
      
      Trf796xReset();
      
      i_reg = 0x02;    // end of RX
      rx_error_flag = 0x02;
    }
    else
    {
      i_reg = 0x02;    // end of RX
    }
  }
  else if((*irq_status & BIT2) == BIT2)    // byte framing error
  {
    if((*irq_status & BIT5) == BIT5)
    {
      i_reg = 0x01;    // RX active
      rx_error_flag = 0x02;
    }
    else
      i_reg = 0x02;    // end of RX
  }
  else if((*irq_status == BIT0))
  {                        // No response interrupt
    i_reg = 0x00;
    
  }
  else
  {                        // Interrupt register not properly set
  }
}                            // Interrupt Service Routine

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

#pragma vector = PORT2_VECTOR
__interrupt void
Trf796xPortB(void)                            // interrupt handler
{
  u08_t irq_status[4];
  
  irq_flag = 0x02;
  
  STOP_COUNTER;                            // stop timer mode
  
  do
  {
    IRQ_CLR;                            // PORT2 interrupt flag clear
    
    Trf796xReadIrqStatus(irq_status);
    
    // IRQ status register has to be read
    
    if(*irq_status == 0xA0)                // TX active and only 3 bytes left in FIFO
    {
      break;
    }
    
    Trf796xISR(&irq_status[0]);
  } while((IRQ_PORT & IRQ_PIN) == IRQ_PIN);
  __bic_SR_register_on_exit(LPM0_bits);
}

//===============================================================
// NAME: void Trf796xRawWrite (u08_t *pbuf, u08_t length)
//
// BRIEF: Is used to write direct to the TRF796x.
//
// INPUTS:
//    Parameters:
//        u08_t        *pbuf        raw data
//        u08_t        length        number of data bytes
//
// OUTPUTS:
//
// PROCESS:    [1] send raw data to TRF796x
//
// CHANGE:
// DATE          WHO    DETAIL
// 24Nov2010    RP    Original Code
//===============================================================

void
Trf796xRawWrite(u08_t *pbuf, u08_t length)
{
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    SpiRawWrite(pbuf, length);
  }
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf796xReConfig(void)
{
  if(SPIMODE)
  {
    SpiUsciExtClkSet();                   // Set the USART
  }
}

//===============================================================
// NAME: void Trf796xReadCont (u08_t *pbuf, u08_t length)
//
// BRIEF: Is used to read a specified number of TRF796x registers
// from a specified address upwards.
//
// INPUTS:
//    Parameters:
//        u08_t        *pbuf        address of first register
//        u08_t        length        number of registers
//
// OUTPUTS:
//
// PROCESS:    [1] read registers
//            [2] write contents to *pbuf
//
// CHANGE:
// DATE          WHO    DETAIL
// 24Nov2010    RP    Original Code
//===============================================================

void
Trf796xReadCont(u08_t *pbuf, u08_t length)
{
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    SpiReadCont(pbuf, length);
  }
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf796xReadIrqStatus(u08_t *pbuf)
{
  *pbuf = IRQ_STATUS;
  *(pbuf + 1) = IRQ_MASK;
  Trf796xReadCont(pbuf, 2);           // read second reg. as dummy read
  
}

//===============================================================
// NAME: void Trf796xReadSingle (u08_t *pbuf, u08_t number)
//
// BRIEF: Is used to read specified TRF796x registers.
//
// INPUTS:
//    Parameters:
//        u08_t        *pbuf        addresses of the registers
//        u08_t        number        number of the registers
//
// OUTPUTS:
//
// PROCESS:    [1] read registers
//            [2] write contents to *pbuf
//
// CHANGE:
// DATE          WHO    DETAIL
// 24Nov2010    RP    Original Code
//===============================================================

void
Trf796xReadSingle(u08_t *pbuf, u08_t number)
{
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    SpiReadSingle(pbuf, number);
  }
  
}

//===============================================================
// resets FIFO                                                  ;
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf796xReset(void)

{
  command[0] = RESET;
  Trf796xDirectCommand(command);
  
}

//===============================================================
// resets IRQ Status                                            ;
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf796xResetIrqStatus(void)
{
  u08_t irq_status[4];
  irq_status[0] = IRQ_STATUS;
  irq_status[1] = IRQ_MASK;
  
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    Trf796xReadCont(irq_status, 2);        // read second reg. as dummy read
  }
  
}

//===============================================================
//                                                              ;
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf796xRunDecoders(void)
{
  command[0] = RUN_DECODERS;
  Trf796xDirectCommand(command);
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf796xStopDecoders(void)
{
  command[0] = STOP_DECODERS;
  Trf796xDirectCommand(command);
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf796xTransmitNextSlot(void)
{
  command[0] = TRANSMIT_NEXT_SLOT;
  Trf796xDirectCommand(command);
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf796xTurnRfOff(void)
{
  command[0] = CHIP_STATE_CONTROL;
  command[1] = CHIP_STATE_CONTROL;
  Trf796xReadSingle(&command[1], 1);
  command[1] &= 0x1F;
  Trf796xWriteSingle(command, 2);
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf796xTurnRfOn(void)
{
  u08_t write[4];
  write[0] = CHIP_STATE_CONTROL;
  write[1] = 0x20;                    // 3.3VDC, Full power out
  Trf796xWriteSingle(write, 2);
  
}

//===============================================================
// NAME: void SpiWriteCont (u08_t *pbuf, u08_t length)
//
// BRIEF: Is used to write to a specific number of reader chip
// registers from a specific address upwards.
//
// INPUTS:
//    u08_t    *pbuf    address of first register followed by the
//                    contents to write
//    u08_t    length    number of registers + 1
//
// OUTPUTS:
//
// PROCESS:    [1] write to the registers
//
// CHANGE:
// DATE          WHO    DETAIL
//===============================================================

void
Trf796xWriteCont(u08_t *pbuf, u08_t length)
{
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    SpiWriteCont(pbuf, length);
  }
}

//===============================================================
// 02DEC2010    RP    Original Code
//===============================================================

void
Trf796xWriteIsoControl(u08_t iso_control)
{
  u08_t write[16];
  
  write[0] = ISO_CONTROL;
  write[1] = iso_control;
  write[1] &= ~BIT5;
  Trf796xWriteSingle(write, 2);
  
  iso_control &= 0x1F;
  write[0] = IRQ_MASK;
  write[1] = 0x3F;
  Trf796xWriteSingle(write, 2);
}

//===============================================================
// NAME: void Trf796xWriteSingle (u08_t *pbuf, u08_t length)
//
// BRIEF: Is used to write to specified reader chip registers.
//
// INPUTS:
//    u08_t    *pbuf    addresses of the registers followed by the
//                    contents to write
//    u08_t    length    number of registers * 2
//
// OUTPUTS:
//
// PROCESS:    [1] write to the registers
//
// CHANGE:
// DATE          WHO    DETAIL
// 24Nov2010    RP    Original Code
//===============================================================

void
Trf796xWriteSingle(u08_t *pbuf, u08_t length)
{
  if(SPIMODE)                                // SPI mode given by jumper setting
  {
    SpiWriteSingle(pbuf, length);
  }
}
